#!/usr/bin/env python

import os, sys, shutil, gzip, tarfile, commands
sys.path.append( "./src" )

def copyfile( srcfile, dstfile ):
    dstdir = os.path.split(dstfile)[0]
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    shutil.copy( srcfile, dstfile )

def copyfiles( srcdir, dstdir ):
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    for file in os.listdir( srcdir ):
        if not os.path.isdir( srcdir + "/" + file ):
            shutil.copy( srcdir + "/" + file, dstdir + "/" + file )

def is_svn_crap( uri ):
    return ( uri.split( "/" )[-1] == ".svn" )

def makewdi( srcdir ):
    destfile = srcdir + ".wdi"
    tf = tarfile.open( destfile, "w" )
    try: 
        tf.add( srcdir, arcname=srcdir.split( "/" )[-1], exclude=is_svn_crap )
    except TypeError:
        tf.add( srcdir, arcname=srcdir.split( "/" )[-1] ) # Python < 2.6
    tf.close()

def findmodule( module ):
    try:
        __import__( module )
        return True
    except:
        return False

def findexec( executable ):
    if commands.getstatusoutput( "which " + executable )[0] == 0:
        return True
    return False

def WineVerGtThan( target_version ):
    # This is so bloody evil
    try:
        wine_ver_string = commands.getoutput( "wine --version" ).split( "-" )[1]
        return wine_ver_string.split( "." ) >= target_version.split( "." )
    except IndexError:
        return False

def copygzip( srcfile, dstfile ):
    dstfile = gzip.GzipFile( dstfile, 'wb' )
    srcfile = open( srcfile )
    dstfile.write( srcfile.read() )
    srcfile.close()
    dstfile.close()
    
def copypacks( repo_root, dstdir ):
    for wdifile in os.listdir( repo_root ): 
        if wdifile.endswith(".wdi"): 
            shutil.copy( repo_root + wdifile, dstdir + wdifile )

def usage( error=None ):
    if error:
        print "Error: " + error
    print "Usage: python setup.py [command] <options>"
    print " Commands:"
    print "   install - installs wine-doors"
    print " uninstall - uninstalls wine-doors"
    print " remove-all - Remove every wine-doors previews version (only use if you know what you are doing)"
    print "    --help - display this usage message (default)\n" 
    print " Options:"
    print "  --prefix - prefix path default ~/.local/ (ex: /usr)"
    #print "  --temp - Used for non final location (if you are not sure don't use it)"
    #print "  -S - make a system install in prefix for packagers"
    #print "  --local-repos - install a local copy of the wine-doors repositories (for development uses only)
    print "  --nodeps - ignore unmet dependancies"
    print
    print " To run winedoors with no install, run 'python src/winedoors.py'"
    sys.exit(0)
    pass
    
def symlink( srclink, dstlink ):
    dstdir = os.path.split(dstlink)[0]
    if not os.path.isdir( dstdir ):
        os.makedirs( dstdir )
    if not os.path.islink( dstlink ):
        os.symlink( srclink, dstlink )

if __name__ == "__main__":
    command = None
    os.chdir(sys.path[0])
    
    # Set Default options (user install):
    wine_doors = "wine-doors" # Can be used to create branches
    prefix = os.path.expanduser( "~/" )
    sysinstall = False
    temp = ""
    nodeps = False
    install_local_repos = False
    
    # Set Defaults case root install
    if os.environ['USER'] == "root":
        prefix = "/usr/" 
        sysinstall = True
        
    # Read command line arguments
    for argument in sys.argv:
        if argument == "install" or argument == "uninstall":
            command = argument
            continue

        if argument == "remove-all":
            command = argument
            continue

        if argument.startswith( "--temp" ):
            temp = argument.split( "=" )[1]
            continue

        if argument == "--nodeps":
            nodeps = True
            continue

        if argument == "--local-repos":
            install_local_repos = True
            continue

        if argument.startswith( "--prefix" ):
            prefix = argument.split( "=" )[1]
            if not prefix.endswith("/"):
                prefix = prefix + "/"
            continue            
                    
        if argument.startswith( "--help" ):
            usage()

        if argument.startswith( "-S" ):
            sysinstall = True
    
    wine_prefix = os.getenv("WINEPREFIX")
    if wine_prefix == None:
        wine_prefix = os.path.expanduser( "~/.wine" )
    
    if sysinstall:
        final = prefix
        prefix = temp + prefix
        prefix_bin = prefix + "/bin/"
        prefix_data = prefix + "/share/"
        prefix_conf = prefix + "/etc/" + wine_doors + "/"
        winedoors_data = prefix_data + wine_doors + "/"

        final_data = final + "/share/wine-doors/"
        final_conf = prefix_conf
    else:
        prefix_bin = prefix + "bin/"
        prefix_data = prefix + ".local/share/"
        prefix_conf = wine_prefix + "/wine-doors/"
        winedoors_data = prefix_data + wine_doors + "/"

        final_data = winedoors_data
        final_conf = prefix_conf
    
    if command in ("install", "uninstall"):
        from preferences import preferences
        
    if command == "install":
        if sysinstall:
            print "Performing system install with parameters: " + final + "  " + prefix

        # Required deps
        missing_required = []

        # Check deps
        print "Checking dependencies"
        print "  wine >= 1.0 . . . ",
        if findexec( "wine" ) and WineVerGtThan( "1.0" ):
            print "Found"
        else:
            print "Not found !"
            missing_required.append("wine")

        print "  cabextract . . . ",
        if findexec( "cabextract" ):
            print "Found"
        else:
            print "Not found !"
            missing_required.append("cabextract")

        print "  orange . . . ",
        if findexec( "orange" ):
            print "Found"
        else:
            print "Not found !"
            missing_required.append("orange")
       
        print "  unzip . . . ",
        if findexec( "unzip" ):
            print "Found"
        else:
            print "Not found !"
            missing_required.append("unzip")

        print "  pgrep . . . ",
        if findexec( "pgrep" ):
            print "Found"
        else:
            print "Not found !"
            missing_required.append("pgrep")

        print "  stat . . . ",
        if findexec( "stat" ):
            print "Found"
        else:
            print "Not found !"
            missing_required.append("stat")

        # Python modules
        print "  python-gtk2 . . . ",
        if findmodule( "gtk" ) and findmodule( "gobject" ):
            hasui = True
            print "Found"
        else:
            print "Not found"
            hasui = False

        print "  python-glade2 . . . ",
        if findmodule( "gtk.glade" ):
            hasui = True
            print "Found"
        else:
            print "Not found"
            hasui = False

        print "  python-cairo . . . ",
        if findmodule( "cairo" ):
            hasui = True
            print "Found"
        else:
            print "Not found"
            hasui = False

        print "  rsvg . . . (python-gnome2-desktop or gnome-python2-rsvg)",
        if findmodule( "rsvg" ):
            hasui = True
            print "Found"
        else:
            print "Not found"
            hasui = False

        if missing_required and not nodeps:
            print
            print "The following dependencies are required to install wine-doors:"
            print missing_required
            sys.exit(1)

        if not hasui:
            print "One of the required modules for the wine-doors user interface is missing"
            print "the wine-doors command line interface is available, for more information" 
            print "try running: " + prefix_bin + "wine-doors --help"
            
        if not os.path.isdir( prefix_conf ):
            os.makedirs( prefix_conf )
        conf_file = prefix_conf + "/preferences.xml"

        # Create directories
        if not os.path.isdir( prefix_data ):
            os.makedirs( prefix_data )
        if not os.path.isdir( winedoors_data ):
            os.makedirs( winedoors_data )
        if not os.path.isdir( prefix_bin ):
            os.makedirs( prefix_bin )
 
        # Build desktop entry
        copyfile( "./dist/wine-doors.desktop",  prefix_data + "/applications/wine-doors.desktop" )
        if os.path.isfile( prefix_data + "/applications/wine-doors.desktop" ):
            desktop = open( prefix_data + "/applications/wine-doors.desktop" , 'a' )
            if sysinstall:
                desktop.write( 'Exec=wine-doors\n' )
                desktop.write( 'Icon=wine-doors\n' )
            else:
                desktop.write( 'Exec=' + prefix_bin + "wine-doors\n" )
                desktop.write( 'Icon=' + winedoors_data + "pixmaps/wine-doors.png\n" )
            desktop.close()

        if install_local_repos:

            # Preparing and install the repositories
            print "Compressing and installing pack files"
            if not os.path.isdir( winedoors_data + "base.repo" ):
                os.makedirs( winedoors_data + "base.repo/" )
            if not os.path.isdir(winedoors_data + "libraries.repo"):
  	        os.makedirs(winedoors_data + "libraries.repo")
	    if not os.path.isdir( winedoors_data + "applications.repo" ):
                os.makedirs( winedoors_data + "applications.repo" )
            if not os.path.isdir( winedoors_data + "games.repo" ):
                os.makedirs( winedoors_data + "games.repo" )

            # Execute scripts to create the packages

            for pkg in os.listdir( "./repos/base.repo/" ):
                if not pkg == ".svn":
                    pkgpath = "./repos/base.repo/" + pkg
                    if os.path.isdir( pkgpath ):
                        makewdi( pkgpath )

            for pkg in os.listdir( "./repos/libraries.repo/" ):
                if not pkg == ".svn":
                    pkgpath =  "./repos/libraries.repo/" + pkg
                    if os.path.isdir( pkgpath ):
                        makewdi( pkgpath )

            for pkg in os.listdir( "./repos/applications.repo/" ):
                if not pkg == ".svn":
                    pkgpath = "./repos/applications.repo/" + pkg
                    if os.path.isdir( pkgpath ):
                        makewdi( pkgpath )

            for pkg in os.listdir( "./repos/games.repo/" ):
                if not pkg == ".svn":
                    pkgpath = "./repos/games.repo/" + pkg
                    if os.path.isdir( pkgpath ):
                        makewdi( pkgpath )
        
            # Install the Repositories
            print "  ** Base Repo"
            copygzip( "./repos/base.repo/packlist.xml", winedoors_data + "base.repo/packlist.xml.gz" )
            copypacks( "./repos/base.repo/", winedoors_data + "base.repo/" )
	
	    print "  ** Libraries.Repo"
            copygzip( "./repos/libraries.repo/packlist.xml", winedoors_data + "libraries.repo/packlist.xml.gz" )
            copypacks( "./repos/libraries.repo/", winedoors_data + "libraries.repo/" )
	
	    print "  ** Applications.Repo"
            copygzip( "./repos/applications.repo/packlist.xml", winedoors_data + "applications.repo/packlist.xml.gz" )
            copypacks( "./repos/applications.repo/", winedoors_data + "applications.repo/" )

            print "  ** Games Repo"
            copygzip( "./repos/games.repo/packlist.xml", winedoors_data + "games.repo/packlist.xml.gz" )
            copypacks( "./repos/games.repo/", winedoors_data + "games.repo/" )
        
        # Install sources
        copyfiles( "./src", winedoors_data + "src" )
        os.system( "chmod +x " + winedoors_data + "src/winedoors.py" )
        copyfiles( "./pixmaps", winedoors_data + "pixmaps/" )
        copyfiles( "./dtd", winedoors_data + "dtd/" )
        copyfiles( "./registry", winedoors_data + "registry/" )

        # install symlinks
        if sysinstall:
            print "Symlinking icons"
            if not os.path.isdir( prefix_data + "/pixmaps/" ):
                os.makedirs( prefix_data + "/pixmaps/" )
            symlink( final_data + "pixmaps/wine-doors.png", prefix_data + "/pixmaps/wine-doors.png" )
            symlink( final_data + "pixmaps/wine-doors.svg", prefix_data + "/pixmaps/wine-doors.svg" )
        
        print "Symlinking executable"
        symlink( final_data + "src/winedoors.py", prefix_bin + "/wine-doors" )
        
        # Fix perms
        if sysinstall:
            print "Setting permissions"
            os.system( "chmod og+r -R "+prefix_data )
        
        # Write preferences.xml
        print "Creating initial preferences"
        preferences.Default()
        preferences.preferences['prefix'] = final_data
        preferences.preferences['wineroot'] = ""

        if install_local_repos:
	    preferences.AddRepo( "Applications", final_data + "applications.repo/" )
            preferences.AddRepo( "System Base", final_data +  "base.repo/" )
            preferences.AddRepo( "Games", final_data +  "games.repo/" )
	    preferences.AddRepo( "Libraries and Fonts", final_data + "libraries.repo/" )
        else:
            preferences.AddRepo( "Applications", "http://www.wine-doors.org/repositories/applications.repo/" )
            preferences.AddRepo( "System Base", "http://www.wine-doors.org/repositories/base.repo/" )
            preferences.AddRepo( "Games", "http://www.wine-doors.org/repositories/games.repo/" )
            preferences.AddRepo( "Libraries and Fonts", "http://www.wine-doors.org/repositories/libraries.repo/" )

        preferences.Save( conf_file )
        preferences.Save( conf_file )
        
    elif command == "uninstall":
        if sysinstall:
            print "Removing system wide preferences"
            if os.path.exists( prefix_conf ):
                shutil.rmtree( prefix_conf )
        
        print "Removing symlinks"
        
        if sysinstall:
            if os.path.exists( prefix_data + "pixmaps/wine-doors.png" ):
                os.remove( prefix_data + "pixmaps/wine-doors.png" )
            if os.path.exists( prefix_data + "pixmaps/wine-doors.svg" ):
                os.remove( prefix_data + "pixmaps/wine-doors.svg" )

        if os.path.exists( prefix_bin + "/wine-doors" ):
            os.remove( prefix_bin + "/wine-doors" )
            
        print "Removing menu item"
        if os.path.exists( prefix_data + "applications/wine-doors.desktop" ):
            os.remove ( prefix_data + "applications/wine-doors.desktop" )
        print "Removing shared data"
        if os.path.isdir( winedoors_data ):
            shutil.rmtree( winedoors_data, True)
            
    elif command == "remove-all":
        if os.path.exists( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.png" )):
            os.remove( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.png" ))
        if os.path.exists( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.svg" )):
            os.remove( os.path.expanduser( "~/.local/share/pixmaps/wine-doors.svg" ))

        if os.path.exists( os.path.expanduser( "~/.wine-doors" )):
            shutil.rmtree( os.path.expanduser( "~/.wine-doors" ), True )

        if os.path.exists( wine_prefix ):
            shutil.rmtree( wine_prefix, True )
            
        print "Removing menu item"
        if os.path.exists( os.path.expanduser( "~/.local/share/applications/wine-doors.desktop" )):
            os.remove ( os.path.expanduser( "~/.local/share/applications/wine-doors.desktop" ))
        if os.path.isdir( os.path.expanduser( "~/.local/share/applications/wine" )):
            shutil.rmtree( os.path.expanduser( "~/.local/share/applications/wine" ))
        print "Removing shared data"
        if os.path.isdir( os.path.expanduser( "~/.local/share/wine-doors" )):
            shutil.rmtree( os.path.expanduser( "~/.local/share/wine-doors" ), True)

        # Removing root install
        if os.path.exists( prefix + "/share/pixmaps/wine-doors.png" ):
            os.remove( prefix + "/share/pixmaps/wine-doors.png" )
        if os.path.exists( prefix + "/share/pixmaps/wine-doors.svg" ):
            os.remove( prefix + "/share/pixmaps/wine-doors.svg" )

        if os.path.isdir( "/etc/wine-doors" ):
            shutil.rmtree( "/etc/wine-doors", True )

        print "Removing menu item"
        if os.path.exists( prefix + "/share/applications/wine-doors.desktop" ):
            os.remove ( prefix + "/share/applications/wine-doors.desktop" )
        print "Removing shared data"
        if os.path.isdir( prefix + "/share/wine-doors" ):
            shutil.rmtree( prefix + "/share/wine-doors", True)

    else:
        usage("No command specified")
